@@ -2168,7 +2168,7 @@ static int  _cr_job_list_sort(void *x, void *y)
 	job_record_t *job1_ptr = *(job_record_t **) x;
 	job_record_t *job2_ptr = *(job_record_t **) y;
 
-	return slurm_sort_uint_list_asc(&job1_ptr->end_time,
+	return slurm_sort_time_list_asc(&job1_ptr->end_time,
 					&job2_ptr->end_time);
 }
 
